#include "mem_test.h"
 
#define PATTERN 0xAA
#define ANTIPATTERN 0x55

#define TEST_PASSED 1
#define TEST_FAILED 0

void printFailure(Xuint32 address, Xuint32 expected, Xuint32 actual);
void printFailureL(Xuint32 address, XuintL expected, XuintL actual);


Xuint8 runDataTestL(Xuint32 address, Xuint32 data_width) {
  Xuint32 i;
  XuintL pattern, antipattern;
  volatile XuintL* test_val = (XuintL*)address;
  XuintL read_val;

  print("Running Data Walking 1's Test...\t");
  for( i = 0, pattern = 1; i < data_width; i++ ) {
    *test_val = pattern;
    read_val = *test_val;
    if( read_val != pattern ) {
      printFailureL((Xuint32)test_val, pattern, read_val);
      return TEST_FAILED;
    }
    pattern += pattern;
  }
  print("SUCCESS!\r\n");

  print("Running Data Walking 0's Test...\t");
  for( i = 0, pattern = 1; i < data_width; i++ ) {
    antipattern = ~pattern;
    *test_val = pattern;
    read_val = *test_val;
    if( read_val != pattern ) {
      printFailureL((Xuint32)test_val, pattern, read_val);
      return TEST_FAILED;
    }    
    pattern += pattern;
  }
  print("SUCCESS!\r\n");

  return TEST_PASSED;

} // end runDataTestL();



Xuint8 runAddrTest(Xuint32 address, Xuint32 addr_width, Xuint32 max_offset) {
  Xuint32 i, j, offset;
  Xuint8 pattern = PATTERN;
  Xuint8 antipattern = ANTIPATTERN;
  volatile Xuint8* test_val = (Xuint8*)address;
  Xuint8 read_val;

  print("Running Address Walking 1's Test...\t");
  for( i = 0; i < addr_width; i++ ) {
    offset = 1 << i;
    test_val[offset] = pattern;
  }

  test_val[0] = antipattern;

  for( i = 0; i < addr_width; i++ ) {
    offset = 1 << i;
    read_val = test_val[offset];
    if( read_val != pattern ) {
      printFailure((Xuint32)test_val+offset, pattern, read_val);
      return TEST_FAILED;
    }      
  }// end for()

  test_val[0] = pattern;

  for( i = 0; i < addr_width; i++ ) {
    Xuint32 test_offset = 1 << i;
    test_val[test_offset] = antipattern;
    read_val = test_val[0];
    if( read_val != pattern ) {
      printFailure((Xuint32)test_val, pattern, read_val);
      return TEST_FAILED;
    }

    for( j = 0; j < addr_width; j++ ) {
      offset = 1 << j;
      read_val = test_val[offset];
      if( (read_val != pattern) && (offset != test_offset) ) {
	printFailure((Xuint32)test_val+offset, pattern, read_val);
	return TEST_FAILED;
      }

    } // end for(j)

    test_val[test_offset] = pattern;
    
  } // end for(i)
  print("SUCCESS!\r\n");

  /** Run the Walking 0's version **/
  print("Running Address Walking 0's Test...\t");
  for( i = 0; i < addr_width; i++ ) {
    offset = ~(1 << i) & max_offset;
    test_val[offset] = pattern;
  }

  test_val[max_offset] = antipattern;

  for( i = 0; i < addr_width; i++ ) {
    offset = ~(1 << i) & max_offset;
    read_val = test_val[offset];
    if( read_val != pattern ) {
      printFailure((Xuint32)test_val+offset, pattern, read_val);
      return TEST_FAILED;
    }      
  }// end for()

  test_val[max_offset] = pattern;

  for( i = 0; i < addr_width; i++ ) {
    Xuint32 test_offset = ~(1 << i) & max_offset;
    test_val[test_offset] = antipattern;
    read_val = test_val[max_offset];
    if( read_val != pattern ) {
      printFailure((Xuint32)test_val+max_offset, pattern, read_val);
      return TEST_FAILED;
    }

    for( j = 0; j < addr_width; j++ ) {
      offset = ~(1 << j) & max_offset;
      read_val = test_val[offset];
      if( (read_val != pattern) && (offset != test_offset) ) {
	printFailure((Xuint32)test_val+offset, pattern, read_val);
	return TEST_FAILED;
      }

    } // end for(j)

    test_val[test_offset] = pattern;
    
  } // end for(i)
  print("SUCCESS!\r\n");

  return TEST_PASSED;

} // end runAddrTest()




Xuint8 runDeviceTestL(Xuint32 address, Xuint32 word_count) {
  Xuint32 i;
  XuintL pattern;
  volatile XuintL* test_val = (XuintL*)address;
  XuintL read_val;

  print("Running Device Pattern 1 Test...\t");
  for( i = 0; i < word_count/2; i++ )
    test_val[i] = i+1;

  for( i = 0; i < word_count/2; i++ ) {
    pattern = i+1;
    read_val = test_val[i];
    if( read_val != pattern ) {
      printFailureL((Xuint32)test_val+(i*8), pattern, read_val);
      return TEST_FAILED;
    }
    test_val[i] = ~pattern;
  } 
  print("SUCCESS!\r\n");

  print("Running Device Pattern 2 Test...\t");
  for( i = word_count/2; i < word_count; i++ )
    test_val[i] = i+1;

  for( i = word_count/2; i < word_count; i++ ) {
    pattern = i+1;
    read_val = test_val[i];
    if( read_val != pattern ) {
      printFailureL((Xuint32)test_val+(i*8), pattern, read_val);
      return TEST_FAILED;
    }
    test_val[i] = ~pattern;
  } 
  print("SUCCESS!\r\n");

  print("Running Device Antipattern 1 Test...\t");
  for( i = 0; i < word_count/2; i++ ) {
    pattern = ~((XuintL)(i+1));
    read_val = test_val[i];
    if( read_val != pattern ) {
      printFailureL((Xuint32)test_val+(i*8), pattern, read_val);
      return TEST_FAILED;
    }
  } // end for(i)
  print("SUCCESS!\r\n");

  print("Running Device Antipattern 2 Test...\t");
  for( i = word_count/2; i < word_count; i++ ) {
    pattern = ~((XuintL)(i+1));
    read_val = test_val[i];
    if( read_val != pattern ) {
      printFailureL((Xuint32)test_val+(i*8), pattern, read_val);
      return TEST_FAILED;
    }
  } // end for(i)
  print("SUCCESS!\r\n");


  return TEST_PASSED;

} // end runDeviceTestL()


void printFailure( Xuint32 address, Xuint32 expected, Xuint32 actual ) {
  print("FAILED!\r\n");
  xil_printf("  Address: 0x%08x, Expected: 0x%08x, Actual: 0x%08x\r\n",
	     address, expected, actual);
} // end printFailure()


void printFailureL( Xuint32 address, XuintL expected, XuintL actual ) {
  XuintL expected_hi = (expected / 4294967296) & 0xFFFFFFFF;
  XuintL actual_hi =  (actual / 4294967296) &  0xFFFFFFFF;
  print("FAILED!\r\n");
  xil_printf("  Address: 0x%08x, Expected: 0x%08x%08x, Actual: 0x%08x%08x\r\n",
	     address, (Xuint32)expected_hi, (Xuint32)expected,
	     (Xuint32)actual_hi, (Xuint32)actual);
} // end printFailureL()


int runTestPatternRGB(int  word_count) {
  unsigned int i, x, wordnum;
  volatile unsigned int rgb_value;
  unsigned int block_start; 
  unsigned int pattern;
  unsigned int test_val;
  unsigned int  *pDisplay_data;

//  pDisplay_data   = (unsigned int  *)0x0FE00000;
  pDisplay_data   = (unsigned int  *)0x07E00000;

  print("\n\rRunning rgb Test Pattern...\t");

  print("...\t");
  x = wordnum = 0;

  // write pattern to DDR
  while (wordnum < word_count)
	{
		// Color pattern
		if (x < 80)
			rgb_value = 0x00FFFFFF; // white
		else if (x < 160)
			rgb_value = 0x00000000; // black
		else if (x < 240)
			rgb_value = 0x00FFFF00; // yellow
		else if (x < 320)
			rgb_value = 0x0000FFFF; // cyan
		else if (x < 400)
			rgb_value = 0x0000FF00; // green
		else if (x < 480)
			rgb_value = 0x00FF00FF; // magenta
		else if (x < 560)
			rgb_value = 0x00FF0000; // red
		else if (x < 640)
			rgb_value = 0x000000FF; // blue

		*(pDisplay_data) = rgb_value;
		pDisplay_data++;
		wordnum++;
		x++;
		if (x >= 640){
		 x = 0;
		 pDisplay_data += (1024 - 640);
		 }

  } // while wordnum < word_count


  print("Passed!\n\r");
  return ;

} // end runTestPatternrgb
